@@ -6,7 +6,7 @@
  * (which requires additional work to test the cache ownership).  This test
  * does not set AUTHTOK.
  *
- * Written by Russ Allbery <rra@stanford.edu>
+ * Written by Russ Allbery <eagle@eyrie.org>
  * Copyright 2011, 2012
  *     The Board of Trustees of the Leland Stanford Junior University
  *
@@ -18,8 +18,6 @@
 
 #include <tests/fakepam/script.h>
 #include <tests/tap/kerberos.h>
-#include <tests/tap/process.h>
-#include <tests/tap/string.h>
 
 
 int
